macroScript VolumePreview category:"Bobo_s Tools"
(
------------------------------------------------------------------
--Volume Preview
--Version 0.33
--Started: 2003/10/16
--Edited:  2003/10/22
--Supports: 3ds max 5, 3ds max 6
--Idea and Code by Borislav 'Bobo' Petrov
--mailto:bobo@email.archlab.tuwien.ac.at
--http://www.scriptspot.com/bobo
------------------------------------------------------------------
--Creates a viewport preview of volumetric effects, 
--shadows and lens effects
------------------------------------------------------------------

	global VolumePreview_SceneID 
	global VolumePreview_generateSceneID
	
	fn VolumePreview_generateSceneID =
	(
		VolumePreview_SceneID = getFileNameFile MaxFileName
	)	

	rollout VolumePreview_rollout "VolumePreview v0.33"
	(
		radiobuttons opacityMode labels:#("RGB Opacity","Alpha") default:2
		button Percent10 "10%" width:30 pos:[10,25]
		button Percent25 "25%" width:30 pos:[40,25]
		button Percent33 "33%" width:30 pos:[70,25]
		button Percent50 "50%" width:30 pos:[100,25]
		button Percent75 "75%" width:30 pos:[130,25]
		button Percent100 "100%" width:30 pos:[160,25]
		edittext actualSize "" align:#left across:2 enabled:false offset:[-7,-1] fieldwidth:98
		spinner sizeFactor "Size %:" range:[1,100,10] type:#integer fieldwidth:30 align:#right

		
		spinner fromFrame "S:" range:[0,10000,animationRange.start] type:#integer fieldwidth:35 align:#left across:3
		spinner toFrame "E:" range:[0,10000,animationRange.end] type:#integer fieldwidth:40 align:#center offset:[2,0]
		spinner byFrame "N:" range:[1,100,1] type:#integer fieldwidth:30 align:#right
		
		button createOverlay "UPDATE ANIMATION RANGE" width:190 height:30 align:#center tooltip:"Generate a Preview using the above Start, End and Nth frame values."
		button renderSingleFrame "UPDATE CURRENT FRAME" width:190 height:30 align:#center tooltip:"Generate a Preview of the current frame only. If no other frames follow, it will be used on following frames."
		
		checkbutton renderEnvironment  "Update Volumetrics"  width:100 align:#left across:2 offset:[-7,0] tooltip:"Generate a Preview of Volumetric Effects."
		checkbutton renderFlares "Update Flares" width:87 align:#right offset:[7,0] tooltip:"Generate Previews of Lens Flares."
		checkbutton renderShadows  "Update Shadows"  width:100 align:#left across:2 offset:[-7,0] tooltip:"Toggle Shadows rendering in the Volumetrics layer."
		checkbutton antiAliasToggle "Anti-Aliasing" width:87 align:#right offset:[7,0] tooltip:"Toggle Anti-Aliasing. Trade off quality vs. speed."
		

--		dropdownlist renderMode items:#("Update Environment Preview","Update Flares Preview","Update Env. & Flares Previews") width:180 align:#center
		
		progressbar progress_display height:10 color:(color 200 0 0 0) width:190 align:#center
		button purgeDisk "Remove Preview"  width:190 align:#center tooltip:"Delete all files from disk and remove the Overlay from the scene."


		
		checkbutton showVFB "Show VFB" width:100 align:#left offset:[-9,0] across:2  tooltip:"Display Virtual Frame Buffer when generating the Preview."
		checkbutton renderableOverlay "Renderable" width:92 align:#right offset:[9,0] tooltip:"When checked, the Overlay will appear in renders. Disable Atmosphere in Renderer to use Overlay only."
		checkbutton enableOverlay "Show Volumetrics" width:100 align:#left across:2 offset:[-9,0] checked:true tooltip:"Toggle the Volumetrics Preview display. "
		checkbutton enableLensFlares "Show Lens Flares" width:92 align:#right offset:[9,0] checked:true tooltip:"Toggle Lens Flares Preview display. "
		
		on enableOverlay changed state do 
		(
			$VolumePreview_Environment*.isHidden = not state
			setIniSetting (getDir #plugcfg + "/VolumePreview.ini") "Settings" "ShowEnvironment" (state as string)
		)
		on enableLensFlares changed state do 
		(
			$VolumePreview_Flare*.isHidden = not state
			setIniSetting (getDir #plugcfg + "/VolumePreview.ini") "Settings" "ShowLensFlares" (state as string)
		)
		
		on renderableOverlay changed state do $VolumePreview*.renderable = state
		
		on opacityMode changed state do 
			setIniSetting (getDir #plugcfg + "/VolumePreview.ini") "Settings" "OpacityMode" (state as string)
		on byFrame changed value do 
			setIniSetting (getDir #plugcfg + "/VolumePreview.ini") "Settings" "NthFrame" (value as string)
		on antiAliasToggle changed state do 
			setIniSetting (getDir #plugcfg + "/VolumePreview.ini") "Settings" "AntiAliasing" (state as string)

		on renderEnvironment changed state do 
			setIniSetting (getDir #plugcfg + "/VolumePreview.ini") "Settings" "RenderEnvironment" (state as string)
		on renderFlares changed state do 
			setIniSetting (getDir #plugcfg + "/VolumePreview.ini") "Settings" "RenderFlares" (state as string)
		on renderShadows changed state do 
			setIniSetting (getDir #plugcfg + "/VolumePreview.ini") "Settings" "RenderShadows" (state as string)
	
		
		fn checkActualSize =
		(
			setIniSetting (getDir #plugcfg + "/VolumePreview.ini") "Settings" "MapSize" (sizeFactor.value as string)
		
			if viewport.getCamera() != undefined then
				actualSize.text = "" + (getViewSize() * sizeFactor.value / 100.0) as string
			else
				actualSize.text = "Not a Camera View!"
		)	
		
		on Percent10 pressed do 
		(
			sizeFactor.value = 10
			checkActualSize()
		)
		on Percent25 pressed do 
		(
			sizeFactor.value = 25
			checkActualSize()
		)
		on Percent33 pressed do 
		(
			sizeFactor.value = 33
			checkActualSize()
		)
		on Percent50 pressed do 
		(
			sizeFactor.value = 50
			checkActualSize()
		)
		on Percent75 pressed do 
		(
			sizeFactor.value = 75
			checkActualSize()
		)
		on Percent100 pressed do 
		(
			sizeFactor.value = 100
			checkActualSize()
		)



		on sizeFactor changed value do checkActualSize()
		on purgeDisk pressed do 
		(
			q = querybox "Are you sure you want to DELETE\nall VolumePreview files from disk?" title:"VolumePreview"
			if q do
			(
				try($VolumePreview*.material = undefined)catch()
				delete $VolumePreview*

				old_files = getFiles (GetDir #preview + "/VolumePreview/VolumePreview_"+ VolumePreview_SceneID +"*.*")		--collect any existing preview images
				old_flare_files = getFiles (GetDir #preview + "/VolumePreview/VolumePreview_Flare_"+ VolumePreview_SceneID +"*.*")		--collect any existing preview images
				
				progress_display.color = color 0 200 0 0
				for f = 1 to old_files.count do 
				(
					progress_display.value = 100.0*f/old_files.count				
					deleteFile old_files[f]								
				)
				progress_display.color = color 255 200 0 0
				for f = 1 to old_flare_files.count do 
				(
					progress_display.value = 100.0*f/old_flare_files.count		
					deleteFile old_flare_files	[f]								
				)
							
				VolumePreview_generateSceneId()
				progress_display.value= 0.0
			)	
		)
		
		fn CreateVolumetricPreview fromFrame toFrame byFrame =
		(
			if viewport.getCamera() != undefined then
			(
				startTime = timestamp()							
				VolumePreview_generateSceneId()					
				maxOps.setActiveViewportTransparencyDisplay 2	
				
				if renderEnvironment.checked or renderShadows.checked do delete $VolumePreview_Environment*		
				if renderFlares.checked do delete $VolumePreview_Flare*		
				
				theViewSize = getViewSize()						
				
				originalMaterials = #()							
				sceneObjects = for o in geometry where classof o != ShagView collect o	
				originalMaterials = for o in sceneObjects collect o.material	
				theMatteMaterial = matteShadow opaqueAlpha:false applyAtmosphere:true atmosphereDepth:1	
				sceneObjects.material = theMatteMaterial 	
				
				oldColorDepth = tgaIo.getColorDepth()			
				oldPreMulti = tgaIo.getPreMultAlpha ()		
				oldAlphaSplit = tgaIo.getAlphaSplit ()		
				tgaIo.setColorDepth 32					
				tgaIo.setPreMultAlpha false				
				
				renderscenedialog.close()					
				oldAspect = renderPixelAspect 			
				renderPixelAspect = 1.0			
				
				outputSize = (theViewSize*sizeFactor.value/100.0)	
				previewbitmap = bitmap outputSize.x outputSize.y	
				if showVFB.checked do display previewbitmap 

				progress_display.color = color 200 0 0 0
				oldRenderEffectsState = RenderEffects
				
				if renderEnvironment.checked or renderShadows.checked do
				(
					RenderEffects = off
					for t = fromFrame to toFrame by byFrame do
					(
						pushprompt ("VolumePreview Env.: Frame "+t as string)
						progress_display.value = 100.0*t/toFrame 		
						theFile = (GetDir #preview + "/VolumePreview/VolumePreview_"+ VolumePreview_SceneID + "_" + t as string +".tga") 	
						if showVFB.checked then													
							render outputSize:outputSize Frame:t outputFile:theFile vfb:false to:previewbitmap antialiasing:antiAliasToggle.state forceWireframe:off shadows:renderShadows.state enablePixelSample:off mapping:off renderatmosphericeffects:renderEnvironment.state
						else	
							render outputSize:outputSize Frame:t outputFile:theFile vfb:false antialiasing:antiAliasToggle.state forceWireframe:off shadows:renderShadows.state enablePixelSample:off mapping:off renderatmosphericeffects:renderEnvironment.state
					)
				)
				
				progress_display.color = color 0 0 200 0
				if renderFlares.checked do
				(
					theMatteMaterial.opaqueAlpha = true
					RenderEffects = on
					for t = fromFrame to toFrame by byFrame do			
					(
						pushprompt ("VolumePreview Flare: Frame "+t as string)
						progress_display.value = 100.0*t/toFrame 		
						theFile = (GetDir #preview + "/VolumePreview/VolumePreview_Flare_"+ VolumePreview_SceneID + "_" + t as string +".tga") 	
						if showVFB.checked then													
							render outputSize:outputSize Frame:t outputFile:theFile vfb:false to:previewbitmap antialiasing:antiAliasToggle.state forceWireframe:off shadows:off enablePixelSample:off mapping:off renderatmosphericeffects:off RenderEffects:on
						else	
							render outputSize:outputSize Frame:t outputFile:theFile vfb:false antialiasing:antiAliasToggle.state forceWireframe:off shadows:off enablePixelSample:off mapping:off renderatmosphericeffects:off RenderEffects:on
					)
				)	
				
				close previewbitmap 													
				RenderEffects = oldRenderEffectsState 
																						
				if renderEnvironment.checked or renderShadows.checked do																	
				(
					new_ifl = createFile (GetDir #preview + "/VolumePreview/VolumePreview_"+VolumePreview_SceneID + ".ifl")				
					existingFiles = getFiles (GetDir #preview + "/VolumePreview/VolumePreview_"+VolumePreview_SceneID + "_" +"*.tga") 	
					last_file = undefined
				)
				if renderFlares.checked do
				(
					new_flare_ifl = createFile (GetDir #preview + "/VolumePreview/VolumePreview_Flare_"+VolumePreview_SceneID + ".ifl")	
					existingFlareFiles = getFiles (GetDir #preview + "/VolumePreview/VolumePreview_Flare_"+VolumePreview_SceneID + "_" +"*.tga") 
					last_flare_file = undefined
				)	
				
				for t = animationrange.start.frame as integer to animationrange.end.frame as integer do	
				(
					if renderEnvironment.checked or renderShadows.checked do																	
					(
						theFile = (GetDir #preview + "/VolumePreview/VolumePreview_"+ VolumePreview_SceneID + "_" + t as string +".tga") 	--build the current name
						if findItem existingFiles theFile == 0 then								
						(
							if last_file == undefined then
							(
								newBmp = bitmap outputSize.x outputSize.y				
								newBmp.filename = theFile 								
								save newBmp												
								close newBmp 											
								format "%\n" (FileNameFromPath theFile) to:new_ifl		
							)
							else
								format "%\n" (FileNameFromPath last_file) to:new_ifl	
						)	
						else
						(
							last_file = theFile 
							format "%\n" (FileNameFromPath theFile) to:new_ifl			
						)	
					)
					if renderFlares.checked do
					(
						theFlareFile = (GetDir #preview + "/VolumePreview/VolumePreview_Flare_"+ VolumePreview_SceneID + "_" + t as string +".tga") 	--build the current name
						if findItem existingFlareFiles theFlareFile == 0 then								
						(
							if last_flare_file == undefined then
							(
								newBmp = bitmap outputSize.x outputSize.y				
								newBmp.filename = theFlareFile 							
								save newBmp												
								close newBmp 											
								format "%\n" (FileNameFromPath theFlareFile) to:new_flare_ifl		
							)
							else
								format "%\n" (FileNameFromPath last_flare_file) to:new_flare_ifl	
						)	
						else
						(
							last_flare_file = theFlareFile 
							format "%\n" (FileNameFromPath theFlareFile) to:new_flare_ifl			
						)	
					)
				)	
				if renderEnvironment.checked or renderShadows.checked do																	
				(
					close new_ifl 			
					thePlane = Plane name:(uniquename "VolumePreview_Environment") lengthsegs:1 widthsegs:1		
					enableOverlay.checked = true									
					theCamera = viewport.getCamera() 								
					thePlane.transform = theCamera.transform						
					in coordsys theCamera thePlane.pos = $Camera01.pos+[0,0,-10]	
					thePlane.parent = theCamera 									
					corner1 = mapScreenToView [0,0] 10.0							
					corner2 = mapScreenToView theViewSize 10.0						
					thePlane.width = abs(corner2 .x - corner1 .x) 					
					thePlane.length = abs (corner1.y - corner2 .y)					
					thePlane.mapCoords = true										
					thePlane.renderable = false										
					thePlane.showFrozenInGray = false								
					thePlane.isfrozen = true										
					thePlane.material = standardMaterial selfillumamount:100.0 diffuse:[255,255,255]	
					thePlane.material.diffusemap = bitmaptexture filename:(getDir #preview + "/VolumePreview/VolumePreview_"+VolumePreview_SceneID + ".ifl")	
					thePlane.material.opacitymap = bitmaptexture filename:(getDir #preview + "/VolumePreview/VolumePreview_"+VolumePreview_SceneID + ".ifl")	
					thePlane.material.opacitymap.monoOutput = opacityMode.state-1	
					showTextureMap thePlane.material true		
					thePlane.isHidden = not enableOverlay.state					
				)

				if renderFlares.checked do																	
				(
					close new_flare_ifl 	
					thePlane = Plane name:(uniquename "VolumePreview_Flare") lengthsegs:1 widthsegs:1		
					enableOverlay.checked = true									
					theCamera = viewport.getCamera() 								
					thePlane.transform = theCamera.transform						
					in coordsys theCamera thePlane.pos = $Camera01.pos+[0,0,-9]	
					thePlane.parent = theCamera 									
					corner1 = mapScreenToView [0,0] 9.0								
					corner2 = mapScreenToView theViewSize 9.0						
					thePlane.width = abs(corner2 .x - corner1 .x) 					
					thePlane.length = abs (corner1.y - corner2 .y)					
					thePlane.mapCoords = true										
					thePlane.renderable = false										
					thePlane.showFrozenInGray = false								
					thePlane.isfrozen = true										
					thePlane.material = standardMaterial selfillumamount:100.0 diffuse:[255,255,255]	
					thePlane.material.diffusemap = bitmaptexture filename:(getDir #preview + "/VolumePreview/VolumePreview_Flare_"+VolumePreview_SceneID + ".ifl")	
					thePlane.material.opacitymap = bitmaptexture filename:(getDir #preview + "/VolumePreview/VolumePreview_Flare_"+VolumePreview_SceneID + ".ifl")	
					showTextureMap thePlane.material true							
					thePlane.isHidden = not enableLensFlares.state
				)	
					
				for o = 1 to sceneObjects.count do sceneObjects[o].material = originalMaterials[o]	
				
				tgaIo.setColorDepth oldColorDepth 								
				tgaIo.setPreMultAlpha oldPreMulti 								
				tgaIo.setAlphaSplit oldAlphaSplit 								
				renderpixelaspect = oldaspect 									
				freeSceneBitmaps()												
				endTime = timestamp()											
				format "VolumePreview Finished in % secs.\n" ((endTime-StartTime)/1000.0)
				pushprompt ("VolumePreview: "+((endTime-StartTime)/1000.0) as string + " secs.")
			)
			else
				messagebox "Active View is NOT a Camera View!" title:"VolumePreview"	
			progress_display.value = 0													
		)	
		
		on createOverlay pressed do
			CreateVolumetricPreview fromFrame.value toFrame.value byFrame.value
			
		on renderSingleFrame pressed do
			CreateVolumetricPreview (SliderTime.frame as integer) (SliderTime.frame as integer) 1
		
		on VolumePreview_rollout moved pos do
			setIniSetting (getDir #plugcfg + "/VolumePreview.ini") "Dialog" "Position" (pos as string)
		
	)
	destroyDialog VolumePreview_rollout
	try(rolout_pos = execute (getIniSetting (getDir #plugcfg + "/VolumePreview.ini") "Dialog" "Position"))catch()
	if rolout_pos == OK then rolout_pos = [100,100]
	createDialog VolumePreview_rollout 200 310 rolout_pos.x rolout_pos.y
	try(VolumePreview_rollout.opacityMode.state = execute (getIniSetting (getDir #plugcfg + "/VolumePreview.ini") "Settings" "OpacityMode"))catch()
	try(VolumePreview_rollout.sizeFactor.value = execute (getIniSetting (getDir #plugcfg + "/VolumePreview.ini") "Settings" "MapSize"))catch()
	try(VolumePreview_rollout.byFrame.value = execute (getIniSetting (getDir #plugcfg + "/VolumePreview.ini") "Settings" "NthFrame"))catch()
	try(VolumePreview_rollout.antiAliasToggle.state = execute (getIniSetting (getDir #plugcfg + "/VolumePreview.ini") "Settings" "AntiAliasing"))catch()
	try(VolumePreview_rollout.enableLensFlares.state = execute (getIniSetting (getDir #plugcfg + "/VolumePreview.ini") "Settings" "ShowLensFlares"))catch()
	try(VolumePreview_rollout.enableOverlay.state = execute (getIniSetting (getDir #plugcfg + "/VolumePreview.ini") "Settings" "ShowEnvironment"))catch()
	try(VolumePreview_rollout.renderEnvironment.state = execute (getIniSetting (getDir #plugcfg + "/VolumePreview.ini") "Settings" "RenderEnvironment"))catch()
	try(VolumePreview_rollout.renderFlares.state  = execute (getIniSetting (getDir #plugcfg + "/VolumePreview.ini") "Settings" "RenderFlares"))catch()
	try(VolumePreview_rollout.renderShadows.state = execute (getIniSetting (getDir #plugcfg + "/VolumePreview.ini") "Settings" "RenderShadows"))catch()

	VolumePreview_rollout.checkActualSize()
	$VolumePreview*.renderable = false
	$VolumePreview_Flare*.isHidden = not VolumePreview_rollout.enableLensFlares.state
	$VolumePreview_Environment*.isHidden = not VolumePreview_rollout.enableOverlay.state
	VolumePreview_generateSceneID()
	makedir (getDir #preview + "/VolumePreview")
	callbacks.removescripts #FilePostOpen id:#VolumePreview
	callbacks.addscript #FilePostOpen "VolumePreview_generateSceneID()" id:#VolumePreview
	OK
)